% The user must input matrices Kbar (state cost matrix), Abar (transition matrix)
% Cbar (control multiplier matrix), J (crossproduct matrix), R (control cost matrix)
% and scalars b (discount factor), c (weight on final period state cost), T
% (horizon), and the output matrix D such that D'*D = [Kbar J; J'% R].  
% The latter is necessary to transform the problem to minimal order.
% Use RedMainFinNoD if the D matrix is not available.
% Note that all variables definitions are case sensitive.
% D:    d by n+k
% Kbar: n by n
% Abar: n by n
% Cbar: n by k
% J: 	n by k
% R: 	k by k
% b:    1 by 1
% c: 	1 by 1
% T: 	1 by 1
% Output is the size of the Riccati kernel and the full feedback and
% Riccati matrices.

KTbar=c*Kbar;
JT=c*J;
RT=c*R;
[nbar,kbar]=size(Cbar);

% Augment the state space with the control variables to put the problem in
% Chow form.
K=[Kbar J; J' R];
KT=[KTbar JT; JT' RT];
A=sqrt(b)*[Abar zeros(nbar,kbar); zeros(kbar,nbar) zeros(kbar)];
C=sqrt(b)*[Cbar; eye(kbar)];
[n,k]=size(C);
'Increase in # of state variables due to adding controls as states:'
Reduction1=nbar-n

% Transform the problem to make it observable.
[Ao, Co, Ko, Vo]=RedObs(A,C,D)
[no,ko]=size(Co);
'Decrease in # of state variables due to nonobservable states:'
Reduction2=n-no

% Transform the problem to make it controllable.  Since it is also
% observable now, the state space is of minimal order.
[Ac, Cc, Kc, S]=RedCon(Ao,Co,Ko)
[nc,kc]=size(Cc);
'Reduction in # of state variables due to noncontrollable states:'
Reduction3=no-nc

% Reduce the Riccati dimension by the number of controls.
timediff=0
[Ak,Kk,Ck,Rk,TransfoK]=RedK(Ac,Cc,Kc,c)
KTk=c*Kk;
[nk,kk]=size(Ck);
if nk<nc
    timediff=timediff+1
end
'Reduction in # of state variables due to R singularities:'
Reduction4=nc-nk

Aq=Ak; Kq=Kk; Cq=Ck; Rq=Rk; KTq=KTk;

nq=nk
kq=kk

Transfo2star=zeros(kq,nq);
Transfo1star=eye(nq);
Tr=zeros(nq);
RoundA=0;

% Reduce the Riccati dimension by exploiting singularities in the A matrix.
% The transformed A matrix may be singular again so keep reducing until A
% is invertible.
atimediff=timediff
while rank(Aq)<nq
    [Ad,Cd,Kd,Rd,Transfo1,Transfo2]=RedA(Aq,Cq,Kq,Rq,c)
    Tr=Transfo1star'*Tr*Transfo1star+Kq
    Aq=Ad; Cq=Cd; Kq=Kd; Rq=Rd;
    Transfo2star=Transfo2*Transfo1 + Transfo2star*Transfo1star;
    Transfo1star=Transfo1*Transfo1star;
    [nq,kq]=size(Cq)
    atimediff=atimediff+1
    'Round of reduction based on A singularities:'
    RoundA=RoundA+1
end
Aa=Aq; Ca=Cq; Ka=Kq; Ra=Rq; KTa=c*Kq;
'Reduction in # of state variables due to A singularities:'
Reduction5=nk-nq

% Apply basic Riccati iteration for Riccati kernel.
[Ha,Fa]=BasicRiccati(Aa,Ca,Ka,KTa,Ra,T);

% Transform Riccati kernel and associated feedback matrix to apply to
% original problem.
for t=1:T-atimediff
    Fk(:,:,t+timediff)=Transfo2star + Fa(:,:,t+atimediff)*Transfo1star;
    Hk(:,:,t+timediff)=Transfo1star'*Ha(:,:,t+atimediff)*Transfo1star + Tr;
    Fc(:,:,t)=Fk(:,:,t+timediff)*TransfoK + inv(Cc'*Kc*Cc)*Cc'*Kc*Ac;
    Hc(:,:,t)=TransfoK'*Hk(:,:,t+timediff)*TransfoK + Kc;
end

% For comparison do basic Riccati iteration for original problem in Chow form.
[H,F]=BasicRiccati(A,C,K,KT,zeros(k),T);

% Generate results for comparing the "big" approach (iteration based on original size)
% to the "small" approach (based on size or Riccati kernel).
for t=1:T-atimediff
    Fbig(:,:,t)=F(:,:,t)*Vo'*S;
    Hbig(:,:,t)=S'*Vo*H(:,:,t)*Vo'*S;
    Fsmall(:,:,t)=Fc(:,:,t);
    Hsmall(:,:,t)=Hc(:,:,t);
end
Fbig
Hbig
Fsmall
Hsmall

'Summary of reduction in # of state variables:'
Reduction1
Reduction2
Reduction3
Reduction4
Reduction5
'Total Reduction in # of state variables for Riccati iteration:'
TotalReduction=Reduction1+Reduction2+Reduction3+Reduction4+Reduction5
KernelSize=nbar-TotalReduction
MinimalOrderNoR=n-(Reduction1+Reduction2+Reduction3)
MinimalOrder=nbar-(Reduction1+Reduction2+Reduction3)

% ending of file RedMainFin.m
% Copyright July 2003, Ronald J. Balvers, West Virginia University.